package com.shifthackz.aisdv1.core.validation.url

import android.util.Patterns
import android.webkit.URLUtil
import com.shifthackz.aisdv1.core.validation.ValidationResult

internal class UrlValidatorImpl : UrlValidator {

    override operator fun invoke(input: String?): ValidationResult<UrlValidator.Error> = when {
        input == null -> ValidationResult(
            isValid = false,
            validationError = UrlValidator.Error.Empty,
        )
        input.trim().isEmpty() -> ValidationResult(
            isValid = false,
            validationError = UrlValidator.Error.Empty
        )
        !input.startsWith(SCHEME_HTTP) && !input.startsWith(SCHEME_HTTPS) -> ValidationResult(
            isValid = false,
            validationError = UrlValidator.Error.BadScheme,
        )
        input.contains(LOCALHOST_IPV4) -> ValidationResult(
            isValid = false,
            validationError = UrlValidator.Error.Localhost,
        )
        !URLUtil.isValidUrl(input) -> ValidationResult(
            isValid = false,
            validationError = UrlValidator.Error.Invalid,
        )
        !Patterns.WEB_URL.matcher(input).matches() -> ValidationResult(
            isValid = false,
            validationError = UrlValidator.Error.Invalid,
        )
        else -> ValidationResult(isValid = true)
    }

    companion object {
        private const val SCHEME_HTTPS = "https://"
        private const val SCHEME_HTTP = "http://"
        private const val LOCALHOST_IPV4 = "127.0.0.1"
    }
}
